#!/usr/bin/perl
use strict ;
use warnings ;
use autodie ;

use Test::More tests => 42;

use Devel::Cover ;
use File::Temp ;
use File::Copy ;
use File::Basename ;
use English qw( -no_match_vars ) ;

use Stanford::Orange::Util
  qw/
    dn_to_cn
    extract_pod
    print_man_page_from_pod
    files_older_than
    get_wallet
    mysql_time
    number_recent_files
    run_command_improved
    slurp
    spew
    tail_file_matches
    tempfile
    trim
    / ;

my ($x, $z) ;

# Test tempfile
$z = tempfile() ;
ok (-f $z, 'tempfile exists') ;
unlink($z) ;
ok (!(-f $z), 'unlinked tempfile does not exist') ;

# Test slurp
$z = tempfile() ;
spew('Hello, my name is Orange. What is yours.', $z) ;
$x = slurp($z) ;
ok ($x, 'slurped a file') ;

eval {
    $x = slurp('/tmp/does/not/exist.txt') ;
} ;
ok ($@, 'slurping a nonexistent file raises an error') ;

# Test the filesize limit
# Try to slurp a too large file
eval {
    $x = slurp($z, 10) ;
} ;
ok ($@, 'slurping a file that is too big raises an error') ;

$x = slurp($z, undef) ;
ok ($x, 'slurp works with filesizelimit set to undef') ;
$x = slurp($z, 0) ;
ok ($x, 'slurp works with filesizelimit set to 0') ;
$x = slurp($z, 100_000_000,
           'slurp works with filesize set larger than actual size of file' ) ;
unlink($z) ;

# Test spew.
$x = time() ;
my $f = tempfile() ;
spew($x, $f) ;
$z = slurp($f) ;
ok ($x eq $z, 'slurped file matches string') ;

eval {
    $z = spew($x, '/tmp/does/not/exist.txt') ;
} ;
ok ($@ =~ m{does\/not}xsm,
    'error thrown when attemptring to spew to a non-reacahable file' );
unlink($f) ;

# Test trim.
$x = ' aaa bb ' ;
ok (trim($x) eq 'aaa bb', 'basic trim passes') ;
ok (trim(q{   }) eq q{}, 'all whitespace trims to empty string') ;
ok (trim(q{}) eq q{}, 'empty string trims to empty string') ;
ok (!defined(trim(undef)), 'undef trims to undef') ;
ok (trim("  \nabc\n\nxyz  \n  ") eq "abc\n\nxyz", 'embedded returns unaffected by trim') ;

# Test mysql_time
ok (mysql_time() =~ m{^\d\d\d\d-\d\d-\d\d[ ]\d\d:\d\d:\d\d}xsm, 'mysql time works') ;

# Test number_recent_files. We have 4 old files. First pass should give us zero.
my $num_recent_files ;

# Set the date on several files to be in the past:
# ##### ## ##### ## ##### ## ##### ## ##### ## ##### ## ##### ## ##### #
my $touch_old_file_fref = sub {
    my ($number) = @_;
    my @cmdx = (
                'touch',
                "./t/number_recent_files/old$number.txt",
                "--date=2012-12-0$number 13:00:00",
        ) ;
    run_command_improved(@cmdx);
} ;
# ##### ## ##### ## ##### ## ##### ## ##### ## ##### ## ##### ## ##### #
$touch_old_file_fref->('1');
$touch_old_file_fref->('2');
$touch_old_file_fref->('3');
$touch_old_file_fref->('4');

$num_recent_files = number_recent_files('./t/number_recent_files', 60 * 60 * 24) ;
ok ($num_recent_files == 0, 'no recent files') ;

# Now add a recent file
my $nrf_dir = './t/number_recent_files' ;

my $z1 = tempfile() ;
my $nrf1 = File::Spec->catfile(($nrf_dir), basename($z1)) ;
File::Copy::copy($z1, $nrf1);
$num_recent_files = number_recent_files('./t/number_recent_files', 60 * 60 * 24) ;
ok ($num_recent_files == 1, 'now we have one recent file') ;

# Make another one
my $z2 = tempfile() ;
my $nrf2 = File::Spec->catfile(($nrf_dir), basename($z2)) ;
File::Copy::copy($z2, $nrf2);
$num_recent_files = number_recent_files('./t/number_recent_files', 60 * 60 * 24) ;
ok ($num_recent_files == 2, 'now we have two recent files') ;

# Now match using the filename regular expression. Create a new file.
my $z3 = tempfile() ;
my $nrf3 = File::Spec->catfile(($nrf_dir), 'fileabc123.txt') ;
File::Copy::copy($z2, $nrf3);

$num_recent_files = number_recent_files('./t/number_recent_files',
                                        60 * 60 * 24, '123') ;
ok ($num_recent_files == 1, 'down to one recent file') ;

unlink($z1);
unlink($z2);
unlink($z3);
unlink($nrf1) ;
unlink($nrf2) ;
unlink($nrf3) ;

# Test dn_to_cn
my %subj_to_cn = (
  '/C=US/ST=California/L=Stanford/O=Stanford University/OU=IT Services MDM/CN=ios-device-test624.stanford.edu'
     => 'ios-device-test624.stanford.edu',
  '/C=US/ST=California/L=Stanford/O=Stanford University/OU=Authentication and Collaboration Solutions/CN=*.ezproxy.stanford.edu'
     => '*.ezproxy.stanford.edu',
  '/C=US/ST=California/L=Stanford/O=Stanford University/OU=IT Services/CN=italertsme-sbx.stanford.edu/emailAddress=torg@stanford.edu',
     => 'italertsme-sbx.stanford.edu',
  '/CN=jimhenson.stanford.edu',
     => 'jimhenson.stanford.edu',
  '/UID=com.apple.mgmt.External.33775e73-cd38-493c-9760-b34df48c669d/CN=APSP:33775e73-cd38-493c-9760-b34df48c669d/C=US',
     => 'APSP:33775e73-cd38-493c-9760-b34df48c669d',
  '/C=US/O=Internet2/OU=InCommon/CN=InCommon Server CA',
     => 'InCommon Server CA',
  'C = US, ST = California, L = Palo Alto, O = Stanford Law School, OU = IT, CN = law.stanford.edu, emailAddress = webmaster@law.stanford.edu' => 'law.stanford.edu',
    ) ;

foreach my $subject (keys %subj_to_cn) {
    my $cn = $subj_to_cn{$subject} ;
    ok ($cn eq dn_to_cn($subject), "extract $cn") ;
}

ok(!defined(dn_to_cn('xyz')), 'dn_to_cn returns undef when no CN found') ;

my $two_cn = '/C=US/CN=InCommon Server CA/O=Internet2/OU=InCommon/CN=InCommon Server CA';

my $error ;

eval {
    dn_to_cn($two_cn) ;
} ;
$error = $@ ;
ok ($error =~ m{passed[ ]in[ ]DN[ ]has[ ]more[ ]than[ ]one[ ]CN}xsm,
    'dn_to_cn raises error when there is more than one CN') ;

eval {
    dn_to_cn(undef) ;
} ;
$error = $@ ;
ok ($error =~ m{passed[ ]in[ ]DN[ ]is[ ]empty[ ]or[ ]undefined}xsm,
    'dn_to_cn raises an error if the DN is undefined') ;

eval {
    dn_to_cn(q{}) ;
} ;
$error = $@ ;
ok ($error =~ m{passed[ ]in[ ]DN[ ]is[ ]empty[ ]or[ ]undefined}xsm,
    'dn_to_cn raises an error if the DN is empty') ;


# get_wallet
eval {
    get_wallet() ;
} ;
$error = $@ ;
ok ($error =~ m{missing[ ]wallet[ ]object[ ]name}xsm,
    'get_wallet raises an error if no wallet name provided') ;

# Test tail_file_matches
my @matching_lines = tail_file_matches('./t/syslog.txt',
                                       qr/debian-sa1|puppet-agent/xsm, 300) ;
ok (scalar(@matching_lines) == 70, 'correct number of matching lines') ;

## files_older_than
my $fot_dir ;

# Test failure modes.
eval {
    files_older_than() ;
} ;
$error = $EVAL_ERROR ;

ok ($error =~ m{missing.required.directory.parameter}xsm,
    'proper failure with no parameters') ;

eval {
    files_older_than('/tmp/') ;
} ;
$error = $EVAL_ERROR ;

ok ($error =~ m{missing.required.age-limit.parameter}xsm,
    'proper failure with no age-limit parameter') ;

# Set up test drectory
$fot_dir = File::Temp->newdir() ;

# Create some files in $fot_dir
my $fot_new_file = 'new.txt' ;
my $fot_old_file = 'old.txt' ;
my $fot_new_dir  = 'new-dir' ;
my $fot_old_dir  = 'old-dir' ;

my $fot_new_file_fp = File::Spec->catdir(($fot_dir), $fot_new_file) ;
my $fot_old_file_fp = File::Spec->catdir(($fot_dir), $fot_old_file) ;
my $fot_new_dir_fp  = File::Spec->catdir(($fot_dir), $fot_new_dir ) ;
my $fot_old_dir_fp  = File::Spec->catdir(($fot_dir), $fot_old_dir ) ;

spew('Hello', $fot_new_file_fp) ;
spew('Hello', $fot_old_file_fp) ;
mkdir($fot_new_dir_fp) ;
mkdir($fot_old_dir_fp) ;

# Set some times on the old files
my $one_day_ago = time() - (24 * 60 * 60) ;
utime(time(), $one_day_ago, $fot_old_file_fp) ;
utime(time(), $one_day_ago, $fot_old_dir_fp) ;

my $thirty_minutes = 30 * 60 ;
my @old_files = sort(files_older_than($fot_dir, $thirty_minutes)) ;
ok (scalar(@old_files) == 2, 'correct number of old files') ;
ok ($old_files[0] eq $fot_old_dir_fp) ;
ok ($old_files[1] eq $fot_old_file_fp) ;

## extract_pod

my $pod = extract_pod('./t/extract-pod.txt');
ok ($pod =~ m{^=pod.*Second.POD.section.*=cut$}xsm, 'pod text start and stop');
ok ($pod !~ m{This.line.should.NOT.be.in.the.POD.output.}xsm, 'non-pod text not in POD output');

eval {
    $pod = extract_pod('./t/extract-bad-pod-no-cut.txt');
} ;
ok ($@ =~ m{^file.ended.before.=cut}xsm, 'no =cut detected');

eval {
    $pod = extract_pod('./t/extract-bad-pod-two-pods.txt');
} ;
ok ($@ =~ m{^parse.error:.already.in.a.POD.section}xsm, 'incorrectly nested =pods detected');

eval {
    $pod = extract_pod('./t/extract-bad-pod-cut-before-pod.txt');
} ;
ok ($@ =~ m{^parse.error:.was.not.in.a.POD.section}xsm, '=cut before =pod found');

